import React, { useEffect } from 'react'
import { ICellViewModel, INotebookViewModel, ICellState, CellType, IKernelSpecs, isParameterCell } from 'common/lib/types.js'
import Output from './output'
import { Input } from './input'
import { store } from '../store'
import { IState } from '../store/reducer'
import { connect } from 'react-redux'
import client from '../socket'
import Importexport from './import-export'
import { getNotebookKernelInfo } from '../store/utils'

interface Props {
    cellVM: ICellViewModel
    notebookVM: INotebookViewModel
    kernels: IKernelSpecs
}

const Cell: React.FC<Props> = ({ cellVM, notebookVM, kernels }) => {
    const shouldRenderOutput = () => {
        return Boolean(cellVM.cell.outputs.length) || cellVM.cell.type === CellType.MARKDOWN
    }

    const renderToolbar = () => {
        return <>
            <div>
                <span>kernel: {kernels.find(kernel => kernel.name === cellVM.cell.kernelName)?.displayName}</span>
                <span> | </span>
                <span>state: {ICellState[cellVM.cell.state]}</span>
                <span> | </span>
                <span>id: <input type="text" style={{ width: '250px' }} defaultValue={cellVM.cell.id} /></span>
            </div>
            <span> kernel: </span>
            <select name="" id="" onChange={onChangeCellLanguage}>
                {kernels.map((kernel, index: number) => <option key={index} value={JSON.stringify({ kernelName: kernel.name, language: kernel.language, backend: kernel.backend })}>{kernel.displayName}</option>)}
            </select>
            <span> run: </span>
            <button onClick={runCell}>run cell</button>
            <button onClick={interruptCell}>interrupt cell</button>
            <span> type: </span>
            <button onClick={onChangeCellType.bind(null, CellType.MARKDOWN)}>markdown</button>
            <button onClick={onChangeCellType.bind(null, CellType.CODE)}>code</button>
            <button onClick={onChangeCellType.bind(null, CellType.PARAMETER)}>parameter(py)</button>
            <span> output: </span>
            <button onClick={onClearCellOutput}>clear</button>
            <br />
            <Importexport cellVM={cellVM} />
        </>
    }

    const renderInput = () => {
        return <Input cellVM={cellVM} onKeyDown={onKeyDown} onInputChange={onInputChange} />
    }

    const onAddCell = () => {
        store.dispatch({ type: 'addCell' })
    }

    const onClearCellOutput = () => {
        store.dispatch({ type: 'clearCellOutput', payload: { cellVM } })
    }

    const onChangeCellType = (type: CellType) => {
        store.dispatch({ type: 'changeCellType', payload: { cellVM, type } })
    }

    const onChangeCellLanguage = (e: React.ChangeEvent<HTMLSelectElement>) => {
        let languageWithBackend = JSON.parse(e.target.value)
        store.dispatch({ type: 'changeCellLanguage', payload: { cellVM, languageWithBackend } })
    }

    const onKeyDown = (ev: React.KeyboardEvent) => {
        if (ev.keyCode === 13 && ev.ctrlKey) {
            runCell()
        }
    }

    const interruptCell = () => {
        client.emit('cell.interrupt', cellVM.cell)
    }

    const runCell = () => {
        if (isParameterCell(cellVM.cell)) {
            let kernelInfo = getNotebookKernelInfo(notebookVM)
            client.emit('cell.run', cellVM.cell, kernelInfo)
        } else {
            client.emit('cell.run', cellVM.cell)
        }
    }

    const onInputChange = (event: React.ChangeEvent<HTMLTextAreaElement>, cellVM: ICellViewModel) => {
        store.dispatch({ type: 'updateCellSource', payload: { cellVM, source: event.target.value } })
    }

    const renderBottomToolbar = () => {
        return (<>
            <button onClick={onAddCell}>add</button>
        </>)
    }

    useEffect(() => {
        console.log('hll')
    }, [])

    return (
        <>
            {renderToolbar()}
            <br />
            {renderInput()}
            <br />
            {shouldRenderOutput() ? <Output cellVM={cellVM} /> : null}
            {renderBottomToolbar()}
        </>
    )
}

export default connect((state: IState) => ({ notebookVM: state.notebookVM, kernels: state.kernels }))(Cell)
